# coding: utf-8
class Admin::SpotsController < Admin::ApplicationController

  # 1ページの表示件数
  PER_PAGE = 20
  skip_before_filter :verify_authenticity_token,
    :only => [:create, :set_capture, :remove_capture, :update, :destroy, :update_categories]
  skip_before_filter :login_required, :only => [:download]
  
  before_filter :load_spot,
    :only => [:show, :edit, :update, :destroy, :download, :set_capture, :remove_capture]
  
  def index
    @spots = Spot.hikawa
                 .order("name")
                 .paginate(:per_page => PER_PAGE, :page => params[:page])
                 
    @page = {:this_page => @spots.current_page, :prev_page => @spots.previous_page, :next_page => @spots.next_page}
    respond_to do |format|
      format.html
      format.json{render :json => {:spots => @spots, :page => @page}}
    end
  end
  
  def search
    @spots = Spot.hikawa
                 .order("name")
                 .near_spot(params[:lat], params[:lon])
                 .paginate(:per_page => PER_PAGE, :page => params[:page])
                 
    @page = {:this_page => @spots.current_page, :prev_page => @spots.previous_page, :next_page => @spots.next_page}
    respond_to do |format|
      format.html
      format.json{render :json => {:spots => @spots, :page => @page}}
    end
  end
  
  def show
    @organization = @spot.organization

    # cookie
    if cookies[:ref_spot_id].blank?
      cookies[:ref_spot_id] = params[:id]
    end
    respond_to do |format|
      format.html
      format.json do
        attr = @spot.attributes
        if !@spot.image.blank? && File.exists?(@spot.image_path)
          url = url_for(:action => :download, :id => @spot.id,
            :only_path => false,
            :protocol => "http")
        else
          url = ""
        end
  
        render :json => attr.merge(:image_path => url)
      end
    end
  end
  
  def download
    send_file(@spot.image_path,
      :type => "image/jpeg",
      :disposition => "inline",
      :filename => "#{@spot.name}の画像"
    )
  end
  
  def new
    @spot = Spot.new
    respond_to do |format|
      format.json do
        render :json => {:organizations => Organization.all.map{|o|o.attributes}}
      end
    end
  end
  
  # クライアントから作成が前提とする。
  def create
    respond_to do |format|
      format.json do
        json = JSON.parse(params["json"])
        @spot = Spot.new(json["spot"])
        @spot.organization_id = Organization.find_by_dir_name(HIKAWA_DIR_NAME).try(:id)
        if @spot.invalid?
          return render :json => @spot.errors.full_messages
        end
        Spot.transaction do
          @spot.save!
        end
        render :json => {:status => "OK", :spot => @spot.attributes}
      end
    end
  end
  
  # 作成したレコードに対して画像を設定する機能
  def set_capture
    respond_to do |format|
      format.json do
        unless params[:file_path]
          return render json: "NG"
        end
        old_image_path = @spot.image_path
        @spot.image = File.basename(params[:file_path].original_filename)
        save_image_path = @spot.image_path
        begin
          Spot.transaction do
            @spot.save!
            Dir.mkdir(@spot.image_dir_path) unless File.exists?(@spot.image_dir_path)
            File.open(save_image_path, "wb") do |f|
              f.write params[:file_path].read
            end
            if File.exists?(old_image_path)
              File.unlink(old_image_path)
            end
          end
          return render :json => "OK"
        rescue
          File.unlink(save_image_path) if File.exists?(save_image_path)
          return render :json => "NG"
        end
      end
    end
  end
  
  def edit
    @organization = @spot.organization

    # cookie
    if cookies[:ref_spot_id].blank?
      cookies[:ref_spot_id] = params[:id]
    end
    respond_to do |format|
      format.html
      format.json do
        render :json => {:spot => @spot.attributes, :organizations => Organization.all.map{|o|o.attributes}}
      end
    end
  end
  
  def update
    @organization = @spot.organization
    
    json = JSON.parse(params["json"])
    spot = json["spot"]
    
    @spot.attributes = spot
    if @spot.invalid?
      respond_to do |format|
        format.html{return render "edit"}
        format.json{render :json => "NG"}
      end
    end
    @spot.save
    respond_to do |format|
      format.html{return redirect_to admin_spot_path(@spot)}
      format.json{render :json => {:status =>"OK"}}
    end
  end
  
  def destroy
    @spot.destroy
      respond_to do |format|
      format.html{redirect_to admin_spots_path}
      format.json do
        render :json => {:status => "OK"}
      end
    end
  end
  
  def remove_capture
    begin
      Spot.transaction do
        image_path = @spot.image_path
        @spot.update_attributes!(:image => "")
        File.unlink(image_path) if File.exists?(image_path)
        respond_to do |format|
          format.html{return redirect_to admin_spots_path}
          format.json{return render :json => {:status => "OK"}}
        end
      end
    rescue => ex
      respond_to do |format|
        format.html{return redirect_to admin_spots_path}
        format.json{return render :json => {:status => "NG"}}
      end
    end
  end
  
  def get_linear_distance
    spot = Spot.new(:latitude => params[:spot_lat], :longitude => params[:spot_longi])
    render :text => spot.distance(params[:lat].to_f, params[:longi].to_f,true)
  end
  
  def category_list
    spot_categories = SpotCategory.where(:spot_id => params[:spot_id])
    categories = Category.where(:parent_id => params[:id] )
    categories_list = categories.map{|c| c.attributes}
   
    respond_to do |format|
      format.json{return render :json => {:categories => categories_list, :selected => spot_categories.map{|c| c.attributes}}}
    end
  end
  
  def update_categories
    json = JSON.parse(params["json"])
    begin
      ActiveRecord::Base.transaction do
        SpotCategory.joins(:category).destroy_all(['spot_id = ? AND parent_id = ?', params[:id], params[:parent_id]])
        json["categories_id"].each do |c|
          SpotCategory.create!(:spot_id => params["id"], :category_id => c.to_i)
        end
      end
      respond_to do |format|
        format.json do
          return render :json => {:status => "OK"}
        end
      end
    rescue
      respond_to do |format|
        format.json do
          return render :json => {:status => "NG"}
        end
      end
    end
   
  end
  
private

  def load_spot
    if params[:id] && Spot.exists?(params[:id])
      @spot = Spot.find(params[:id])
    else
      return redirect_to admin_spots_path
    end
  end
end
